##############################################################################
# (C) Copyright 2016 Hewlett Packard Enterprise Development LP.
# Copyright 2013-2015 Hewlett-Packard Development Company, L.P.
# @(#) Metrocluster SRDF Module Script.
# @(#) Product Name                :  HPE Metrocluster SRDF
# @(#) Product Version             :  B.12.00.60
#
# *** Note: This file MUST NOT be edited. *****
#
# Any changes made to it will be overwritten when you upgrade to the
# next release of HPE Serviceguard.
#
# Changing this file may lead to unrecoverable package startup
# and/or shutdown problems.
#
#############################################################################
#
#
. /etc/cmcluster.conf
sg_version=`$SGSBIN/cmversion | cut -d "." -f 2-4`
SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
export SG_UTILS

if [ -e /usr/lib/udev/scsi_id ]; then 
    SCSI_ID_PATH="/usr/lib/udev" 
else 
    SCSI_ID_PATH="/sbin"
fi

PVDISPLAY=/sbin/pvdisplay
if [ -f /etc/redhat-release ]; then
    DISTRO="REDHAT"

    ver=`/bin/cat /etc/redhat-release | awk -F "[ .]" '{print $7}'`
    if [[ $ver -eq 5 ]]; then
        PVDISPLAY=/usr/sbin/pvdisplay
    fi
else
    DISTRO="SUSE"
fi

exit_val=0

#
# Source in the utility functions used by the module scripts
#
if [[ -f ${SG_UTILS} ]]; then
    . ${SG_UTILS}
    if (( $? != 0 ))
    then
        echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
        exit 1
    fi
else
    echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
    exit 1
fi


###################################################################
#
# Get the environment for this package through utility function
# sg_source_pkg_env().
#
###################################################################

sg_source_pkg_env $*


###############################################################################
# This function verifies whether the SG-CFS package is running in the cluster.
# Internal Use only
#
#
# Arguments: No arguments
#
#
#
# Returns:    0 - the SG-CFS Package is running in the cluster
#             exits if the SG-CFS Package is not running in the cluster
#
###############################################################################


function checkSGCFS
{
   typeset cfspkgstatus;
   cfspkgstatus=`cmviewcl -f line -p SG-CFS-pkg | awk -F'=' '$1 == "status" {print $2}'`
   if [ $? != 0 ]; then
      sg_log 0 "\tERROR: Function checkSGCFS"
      sg_log 0 "\tERROR: Failed to get the SG-CFS-pkg system multi-node package status"
      exit 1
   fi
   if [ "$cfspkgstatus" != "up" ]; then
      sg_log 0 "\tERROR: Function SGCFS"
      sg_log 0 "\tERROR: The SG-CFS-pkg system multi-node package is not up"
      exit 1
   fi
   sg_log 0 "\tINFO: SG-CFS-pkg system multi-node package is up"
   return 0

}

##################################################################################
# This function checks whether the volume manager is cvm
# Internal Use only
#
#
# Arguments: No arguments

#
#
# Returns:  0 - cvm is used
#           1 - cvm is not used
#
###############################################################################



function checkcvmused
{

   if [[ ${SG_CVM_DG[*]} != "" ]]; then

      if [[ ${SG_CVM_ACTIVATION_CMD} = "" ]]; then
         sg_log 0 "\tERROR: Function checkcvmused"
         sg_log 0 "\tERROR: CVM Activation Command can not be empty while using CVM group"
         exit 1
      fi
      checkSGCFS
      if [ $? -ne 1 ]; then
         return 0
      fi
   fi
   return 1
}

###############################################################################
# This function checks whether the volume manager is vxvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - vxvm is used
#          1 - vxvm is not used
#
###############################################################################


function  checkvxvmused
{
   if [[ ${SG_VXVM_DG[*]} != "" ]]; then
      return 0
   fi
   return 1
}

####################################################################
# Checks whether a device is a DM multipath device.
# Any device in format /dev/mapper/XXXX is a multipath device.
# On RHEL 5.x, device can also be in format /dev/mpath/mpathX
# For udev, we do a readlink and if its in dm-x format, it is a
# multipath device.
####################################################################

function check_if_multipath
{
    typeset device=$1

    echo $device | grep -q -e "/dev/mapper/" -e "/dev/mpath/"
    if [ $? -eq 0 ]; then
        return 0
    else
        udev_tmp=`readlink $device`
        if [ $? -eq 0 ]; then
            echo $udev_tmp | grep -q "dm-"
            if [ $? -eq 0 ]; then
                return 0
            fi
        fi
        return 1
    fi
}



##################################################################################
# This function checks whether the disks belonging to a LVM or SLVM are being
# replicated.
#
# Internal Use only
#
#
# Arguments: None
#
#
# Returns: None
#
###############################################################################


function lvm_disks_replicated
{
    unset list_of_devices
    for vg in ${list_of_vgs[@]} ; do
       # retrieve the device info from lvmadm command
       list_of_devices_temp=$($PVDISPLAY 2>> /dev/null | awk -v vg=$vg '/VG Name/ { if ($3 == vg ) print pline}{pline=$3}')
       list_of_devices=("${list_of_devices[@]}" "${list_of_devices_temp[@]}")
    done
    for device in ${list_of_devices}; do
       srdf_get_disk_info ${device} ${vg} "lvm"
    done
}


##################################################################################
# This function checks whether the disks belonging to a VXVM or CVM diskgroup are
# being replicated.
#
# Internal Use only
#
#
# Arguments: None
#
#
# Returns: None
#
###############################################################################

function vxvm_disks_replicated
{
    for vg in ${list_of_dgs[@]}; do
        list_of_devices=$(vxdisk -s list | awk -v vg=$vg '
                         /^Disk:/   { disk = $2 }
                         /^dgname:/ { if (vg == $2) { print disk; } }')
        list_of_devices=`echo ${list_of_devices} | sed 's/\n/ /g'`
   done
    for device in ${list_of_devices}; do
            # get the persistent dsf equivalent only if the device is in legacy dsf
                disk_in_pers_dsf="/dev/"${device}
                srdf_get_disk_info ${disk_in_pers_dsf} ${vg} "vxvm"
   done
}


###############################################################################
# This function checks whether the volume manager is lvm or slvm
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - lvm is used
#          1 - lvm is not used
#          2 - slvm is used
#
###############################################################################

function checklvmused
{
   if [[ ${SG_VG[*]} != "" ]]; then

      if [[ ${SG_VGCHANGE_CMD} = "" ]]; then
         sg_log 0 "\tERROR: Function checklvmused"
         sg_log 0 "\tERROR: LVM Activation Command can not be empty while using LVM group"
         exit 1
      fi

      if [[ "${SG_VGCHANGE_CMD#*-a}" = " s" ]]; then
         return 2;
      fi

      return 0
   fi
   return 1
}


##################################################################################
# This function checks whether the disk passed as an argument is being replicated
# by srdf.
#
# Internal Use only
#
#
# Arguments: Disk Path , LVM/VXVM groupname, lvm/vxvm
#
#
# Returns:  None
#
###############################################################################
function srdf_get_disk_info
{
    disk_in_pers_dsf=$1
    vg=$2
    ret=0
    if [[ "$3" = "lvm" ]]; then
        type_of_dg="volume group"
    else
        type_of_dg="disk group"
    fi

    check_if_multipath $disk_in_pers_dsf
    if [[ $? -eq 0 ]]; then
        rawDev=`echo ${disk_in_pers_dsf} | awk -F'/' '{print $4}'`
        rawDevList=`multipath -ll $rawDev | grep sd | sed -n 's/[^0]*0//p' | awk '{print $2}' | grep sd`
        for dev in ${rawDevList[@]};do
            cmd_output=`$SGROOT/bin/SRDFGetDiskInfo ${dev} $SRDF_PATH`
            if [[ $? -eq 0 ]]; then
                ret=$?
                break
            else
                ret=$?
                continue
            fi
        done
    else
        if [[ $ver -eq 5 ]]; then
            output=`echo ${disk_in_pers_dsf} | sed 's/dev/block/g'`
            wwn=`"$SCSI_ID_PATH"/scsi_id -g -s "${output}"`
        else
            wwn=`"$SCSI_ID_PATH"/scsi_id --whitelisted "${disk_in_pers_dsf}"`
        fi

# Holds the list of srdf devices which are not configured with multipath
        SCSI_ID_CMD=`lsscsi | grep EMC | grep /dev/ | awk -F/ 'NF>0{print "/dev/"$NF}'`
        for output in ${SCSI_ID_CMD[@]}; do
# if the running node is RHEL5 then use the -g -s flag. --whitelisted flag
# is not supported in RHEL5 - QXCR1001276800
            if [[ $ver -eq 5 ]]; then
                output=`echo $output | sed 's/dev/block/g'`
                temp_wwn=`"$SCSI_ID_PATH"/scsi_id -g -s "${output}"`
            else
                temp_wwn=`"$SCSI_ID_PATH"/scsi_id --whitelisted "${output}"`
            fi
            if [[ "$temp_wwn" = "$wwn" ]]; then
                dev_name=$(/bin/basename $output)
                if [[ $? -eq 0 ]]; then
                    cmd_output=`$SGROOT/bin/SRDFGetDiskInfo /dev/${dev_name} $SRDF_PATH`
                    if [[ $? -eq 0 ]]; then
                        ret=$?
                        break
                    else
                        ret=$?
                        continue
                    fi
                fi
            fi
        done
    fi
    if [[ $ret -ne 0  ]]; then
         warnings[$warncount]="Warning: Unable to find information for Disk ${disk_in_pers_dsf} \
            that belongs to ${vg} configured in this Metrocluster package from symapi db."
        (( warncount=warncount+1 ))
    else
        dg_name=`echo $cmd_output | awk -F":" '{print $2}'`
        if [[ ${dg_name} = "NA" ]]; then
            warnings[$warncount]="Warning: Disk ${disk_in_pers_dsf} belonging to \
                ${type_of_dg} ${vg} is not a part of the device group ${SRDF_DEVICE_GROUP} \
                configured in this Metrocluster package. Ignore this warning if this
                is being done intentionally; else add this disk to the device group ${SRDF_DEVICE_GROUP}"
            (( warncount=warncount+1 ))
         elif [[ ${dg_name} != ${SRDF_DEVICE_GROUP} ]]; then
            if [[ ${internal_use} -eq 1 ]]; then
                errors[$errcount]="Error: Disk ${disk_in_pers_dsf} in the ${type_of_dg} \
                    ${vg} belongs to a different device group ${dg_name} \
                    instead of the ${SRDF_DEVICE_GROUP} that is mentioned in the package configuration file."
            else
                errors[$errcount]="Error: Disk ${disk_in_pers_dsf} in the ${type_of_dg} \
                    ${vg} belongs to a different device group ${dg_name} instead \
                    of the ${SRDF_DEVICE_GROUP} that is mentioned in the package configuration file."
            fi
            (( errcount=errcount+1 ))
         fi
    fi
}

###############################################################################
# This function checks whether given module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - module is used
#          1 - module is not used
#
###############################################################################

function checkifmodused
{
   typeset used_module=$1;
   exit_val=1
   typeset module_name;
   typeset module;

   module_name=`echo "${SG_MODULE_NAME[@]}"`

   for module in $module_name;do
      if [ "$module" = "$used_module" ]; then
         exit_val=0
         break
      fi
   done

   return $exit_val

}


###############################################################################
# This function returns the version of the given module
# Internal Use only
#
#
# Arguments: The name of the module
#
#
#
# Returns: The version of the module
#
###############################################################################

function find_module_version
{
   typeset used_module=$1;
   typeset module_name;
   let counter=0
   typeset module;
   typeset module_location;
   module_name=`echo "${SG_MODULE_NAME[@]}"`

   for module in $module_name;do
      if [ "$module" = "$used_module" ]; then
         module_location=$counter
         break
      else
         let counter=counter+1
      fi
   done
   return ${SG_MODULE_VERSION[${module_location}]}


}

###############################################################################
# This function checks validates the vgs/dgs in Metrocluster configuration
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - Validation succeeded
#          1 - Validation failed
#
###############################################################################

function validate_rep
{
    $SRDF_PATH/symcfg discover &>/dev/null
    ret=$?
    if (( $ret != 0 )); then
        echo "ERROR: SRDF symcli command \"symcfg discover\" failed. Unable to discover the new devices."
        return 1
    fi

    checklvmused ${SG_VG[*]}
    ret=$?
    if (( $ret == 0 )) || (( $ret == 2 )) ; then
        list_of_vgs="${SG_VG[@]}"
        lvm_disks_replicated
    fi

    vxvmused=0
    checkvxvmused ${SG_VXVM_DG[*]}
    if [ $? -ne 1 ]; then
        vxvmused=1
    fi

    checkcvmused ${SG_CVM_DG[*]}
    if [ $? -ne 1 ]; then
        vxvmused=1
    fi

    if [[ ${vxvmused} -eq 1 ]]; then
        # no separate treatment for cvm and vxvm disk
        # merge the dgnames into a common array.
        list_of_dgs="${SG_VXVM_DG[@]} ${SG_CVM_DG[@]}"
        vxvm_disks_replicated

    fi

    i=0
    while [ $i -lt ${#errors[@]} ]; do
        echo ${errors[$i]}
        (( i = i + 1 ))
    done

    i=0

    while [ $i -lt ${#warnings[@]} ]; do
        echo ${warnings[$i]}
        (( i = i + 1 ))
    done

    if  [[ ${errcount} -gt 0 ]]; then
        return 1
    else
        return 0
    fi



}

##################################################################################
# This function returns the list of nodes that are configured to run this package
#
# Internal Use only
#
# Arguments: None
#
# Returns: node list
#
###############################################################################

function get_package_nodes
{
    if (( ${#SG_NODE_NAME[*]} == 1 )); then
        if [ "${SG_NODE_NAME[0]}" == "*" ]; then
            # All the cluster nodes
            package_node_names=`echo "$cmviewclvfline" | awk '/^node:[^|]*\|name=/' | cut -d "=" -f 2`
        fi
    else
        # Subset of cluster nodes
        package_node_names=`echo "${SG_NODE_NAME[@]}"`
    fi
    echo "$package_node_names"
}

##################################################################################
# This function returns the UUID of node name passed.
#
# Internal Use only
#
# Arguments: node name
#
# Returns: node list
#
###############################################################################

function get_uuid_from_node_name
{
    uuid=""
    node_name=$1
    uuid=`echo "$cmviewclvfline" | grep $node_name | awk -F= '/^node:[^|]*\|uuid=/ {print $2}'`
    echo $uuid
}


##################################################################################
# This function verifies the VMFS details.
#
# Internal Use only
#
# Arguments: No Arguments
#
# Returns: 0 - Validation passed
#          1 - Validation failed
#
###############################################################################

function verify_vmfs_details
{
    typeset eligible_nodes_vmfs=""
    typeset cmviewclvfline=""
    typeset ro_nodes=""
    typeset -i disk_flag=0
    typeset devs_rw=""
    typeset disk_wwid_rw_array=()
    typeset devs_ro=""
    typeset disk_wwid_ro=""

    cmviewclvfline=`$SGSBIN/cmviewcl -vf line -s config`
    package_nodes=$(get_package_nodes)
    for node in ${package_nodes[*]}
    do
        output=$($SGSBIN/cmdo -n ${node} "$SRDF_PATH/symdg show $SRDF_DEVICE_GROUP -output XML")
        if [[ $? -ne 0 ]]; then
            if [[ "$output" = *"$SRDF_DEVICE_GROUP"* ]]; then
                echo "ERROR: Command \"symdg show $SRDF_DEVICE_GROUP\" failed at node ${node}."
            else
                echo "ERROR: Serviceguard is unable to communicate with node: \"${node}\". So, unable to execute \"symdg\" command with parameters at this node."
            fi
            return 1
        fi
        echo "${output}" | grep -e "<status>" -e "</status>" | awk -F">" '{print $2}' | awk -F"</" '{print $1}' | grep "Ready" &> /dev/null
        if [[ $? -eq 0 ]]; then
            eligible_nodes_vmfs="$eligible_nodes_vmfs ${node}"
        else
            ro_nodes="$ro_nodes ${node}"
        fi
    done

    eligible_nodes_vmfs=(${eligible_nodes_vmfs})
    ro_nodes=(${ro_nodes})

    # Pass the eligible nodelist to Master Control Script
    echo "export SFC_MC_READ_WRITE_NODELIST=\"${eligible_nodes_vmfs[*]}\"" >$SG_SRC_FRM_CHILD
    echo "export SFC_MC_READ_ONLY_NODELIST=\"${ro_nodes[*]}\"" >>$SG_SRC_FRM_CHILD

    cur_hostname=${SG_HOST_NAME%%.*}

    # Verification is done only from one node.
    if [[ ${cur_hostname} = ${eligible_nodes_vmfs[0]} ]]; then

        eligible_node_uuid=$(get_uuid_from_node_name ${eligible_nodes_vmfs[0]})
        ro_node_uuid=$(get_uuid_from_node_name ${ro_nodes[0]})

        # Getting VMFS details
        vmfs_details=$($SGSBIN/cmvmdiskquery -n $eligible_node_uuid -n $ro_node_uuid )
        if [[ $? -ne 0 ]]; then
            echo "ERROR: Unable to query the ESX Host or VCenter."
            return 1
        fi

        ## Verifying the RW site.
        devs_rw=`$SRDF_PATH/symdg show $SRDF_DEVICE_GROUP -output XML | grep -e "<dev_name>" -e "</dev_name>"  | awk -F">" '{print $2}' | awk -F"</" '{print $1}'`

        # Verifying that all the disks in the SRDF is part of the Package.
        for rw_device in ${devs_rw}
        do
            disk_flag=0
            disk_wwid_rw=`$SRDF_PATH/symdev show $rw_device | grep "Device WWN" | awk NR==1'{print $4}'`
            disk_wwid_rw_array+=("$disk_wwid_rw")
            for ds_name in ${SG_DATASTORE_NAME[*]}
            do
                echo "$vmfs_details"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|grep -i "$disk_wwid_rw" >/dev/null 2>&1
                if [[ $? -eq 0 ]]; then
                    disk_flag=1
                fi
            done
            if [[ $disk_flag -eq 0 ]]; then
                echo "WARNING: The disk with WWID $disk_wwid_rw, configured as part of the RDF Group $SRDF_DEVICE_GROUP"
                echo "      is not part of any of the Datastore specified in the package."
            fi
        done

        # Verifying that all the disk in the Data Store is part of SRDF
        disk_flag=0
        for ds_name in ${SG_DATASTORE_NAME[*]}
        do
            ds_wwids=`echo "${vmfs_details}"|grep -i "VirtualMachine:${eligible_node_uuid}|Datastore:${ds_name}|wwn="|cut -d '=' -f2`
            for wwid in ${ds_wwids}
            do
                echo "${disk_wwid_rw_array[@]}"|grep -i "${wwid##*.}" >/dev/null 2>&1
                if [[ $? -ne 0 ]]; then
                    echo "ERROR: The disk with WWID ${wwid##*.}, used for the Datastore ${ds_name} is not part of the RDF Group $SRDF_DEVICE_GROUP"
                    disk_flag=1
                fi
            done
        done
        if [[ $disk_flag -eq 1 ]]; then
            return 1
        fi

        ## Verifying the RO site.
        devs_ro=`$SGSBIN/cmdo -n ${ro_nodes[0]} $SRDF_PATH/symdg show $SRDF_DEVICE_GROUP -output XML | grep -e "<dev_name>" -e "</dev_name>"  | awk -F">" '{print $2}' | awk -F"</" '{print $1}'`
        if [[ $? -ne 0 ]]; then
            if [[ "$output" = *"$SRDF_DEVICE_GROUP"* ]]; then
                echo "ERROR: Command \"symdg show $SRDF_DEVICE_GROUP\" failed at node ${node}. So, unable to execute \"symdg\" command with parameters at this node."
            else
                echo "ERROR: Serviceguard is unable to communicate with node: \"${node}\""
            fi
            return 1
        fi
        disk_flag=0

        # Verifying that all the disks in the RCVG is part of the Package.
        for ro_device in ${devs_ro}
        do
            disk_wwid_ro=`$SGSBIN/cmdo -n ${ro_nodes[0]} $SRDF_PATH/symdev show $ro_device | grep "Device WWN" | awk NR==1'{print $4}'`
            if [[ $? -ne 0 ]]; then
                if [[ "$output" = *"Invalid device name"* ]]; then
                    echo "ERROR: Command \"symdev show $ro_device\" failed, as the \"$ro_device\" is an invalid device name." 
                else
                    echo "ERROR: Serviceguard is unable to communicate with node: \"${node}\". So, unable to execute \"symdev\" command with parameters at this node."
                fi
                return 1
            fi
            echo "$vmfs_details"|grep -i "VirtualMachine:${ro_node_uuid}|Disk:" |grep -i $disk_wwid_ro >/dev/null 2>&1
            if [[ $? -ne 0 ]]; then
                echo "WARNING: The disk with WWID $disk_wwid_ro, configured as part of the RDF Group $SRDF_DEVICE_GROUP"
                echo "         in the WD site is not available to the host. Package will not start after fail-over to WD site."
            fi
        done
    fi

    return 0    
}

###############################################################################
# This function checks whether ccxxx module is included or not
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - ccxxx module is used
#          1 - ccxxx module is not used
#
###############################################################################

function check_cc_module
{
        exit_val=1
        typeset module_name;
        typeset module;

        module_name=`echo "${SG_MODULE_NAME[@]}"`

        for module in $module_name;do
            if [[ "$module" = "dts/ccxpca" || "$module" = "dts/cccaeva" || \
                       "$module" = "dts/ccsrdf" ]]; then
                exit_val=0
                break
            fi
        done

    return $exit_val
}


function rescan_devices
{
    typeset wwids=""
        if [[ "$SRDF_CONSISTENCYGROUPS" == "1" ]]; then
            dg_devs=`$SRDF_PATH/symcg show $SRDF_DEVICE_GROUP -output XML 2>/dev/null | grep -v "Not Visible"| grep -e "<pd_name>" -e "</pd_name>" | awk -F">" '{print $2}' | awk -F"</" '{print $1}'`
            for dev in ${dg_devs[@]};do
               wwid=`$SRDF_PATH/sympd show $dev 2>/dev/null | grep "Device WWN" | awk -F":" '{print $2}' | tr -d " "` 
               wwids=("${wwids[@]}" $wwid)
            done
        else
            dg_devs=`$SRDF_PATH/symdg show $SRDF_DEVICE_GROUP -output XML 2>/dev/null | grep -v "Not Visible"| grep -e "<pd_name>" -e "</pd_name>" | awk -F">" '{print $2}' | awk -F"</" '{print $1}'`
            for dev in ${dg_devs[@]};do
               wwid=`$SRDF_PATH/sympd show $dev 2>/dev/null | grep "Device WWN" | awk -F":" '{print $2}' | tr -d " "` 
               wwids=("${wwids[@]}" $wwid)
               done
        fi
        for disk in ${wwids[@]}; do
               disk=`echo $disk | awk '{print tolower($0)}'`
               $SGROOT/bin/mcsrdf/mcsrdf_multipath_rescan.sh 3${disk}
        done
            sleep 10 # sleeping for rescan to take effect
}

###############################################################################
# This function invokes the metrocluster srdf binary
# Internal Use only
#
#
# Arguments:  No Arguments
#
#
#
# Returns: 0 - metrocluster srdf operation is successful
#         exit - 0 - metrocluster srdf operation is not successful
#
###############################################################################

function do_drif
{
        for vg in ${SG_VG[*]};do
            typeset hostid=""

            hostid=$(vgs --noheadings -o +tags $vg | awk  '{print $8}')
            if [[ "$hostid" = *,* ]]; then
                sg_log 0 "ERROR: Volume Group $vg has multiple tags \"($hostid)\" defined."
                sg_log 0 "There cannot be more than one tag."
                sg_log 0 "Remove the tags using the command:"
                for i in ${hostid}; do
                    sg_log 0 "\"vgchange --deltag $i $vg\"."
                done
                return 1
            fi

            if [[ "$hostid" != "" ]] ; then
            #   cmexec $hostid vgchange -a n $vg
                vgchange --deltag $hostid $vg
            fi
        done
        echo "export SFC_MC_RESCAN_REQUIRED=\"1\"" >$SG_SRC_FRM_CHILD
        for vg in "${SG_VXVM_DG[@]}";do
            vxdg -tfC import $vg
        done
        return 0
}


#######################################################################
#
# function sg_start_srdf() is called by master control script
# during metrocluster srdf package startup.
#
#######################################################################


function sg_start_srdf
{
	typeset envfile;
	typeset count;
	typeset ret;
	typeset exit_val;
    sg_log 0 "Starting Metrocluster SRDF Package"
	checklvmused ${SG_VG[*]}
	 ret=$?;
	if [ $ret -eq  0  ]; then
	        lvmisused=1
	
	elif [ $ret -eq 2 ]; then
	        lvmisused=2
	elif [ $ret -eq 1 ]; then
		lvmisused=0
	fi

	checkvxvmused ${SG_VXVM_DG[*]}
	if [ $? -ne 1 ]; then
	        vxvmisused=1
	else
	        vxvmisused=0
	fi

	checkcvmused ${SG_CVM_DG[*]}
	if [ $? -ne 1 ]; then
	        cvmisused=1
	else
	        cvmisused=1
	fi
	if [[ -f $SGROOT/bin/DRCheckSRDFDevGrp ]]; then
	
		if (( $cvmisused == 0 )); then
			 
 			$SGROOT/bin/DRCheckSRDFDevGrp $SG_PACKAGE_NAME 0 0 "" ""
		else
	 		$SGROOT/bin/DRCheckSRDFDevGrp $SG_PACKAGE_NAME 0 0 0 "" "" "" 
			
		fi

			exit_val=$?
	
	else

        echo "ERROR: Cannot find the file: DRCheckSRDFDevGrp file. Exiting."
	exit 1
	
	fi

    if [[ $exit_val -eq 0 ]]; then
        rescan_devices
    fi

	if [[ $exit_val -eq 3 ]]; then # Storage switched from read-only to read-write
        rescan_devices
        do_drif
        exit_val=$?
        if [[ $exit_val -eq 1 ]]; then
	        exit_val=1
            exit $exit_val
	    fi
    fi
	
	if [[ $exit_val -ne 0 ]]; then
	    exit $exit_val
	fi
	to_exit=0
    exit $to_exit
}

#######################################################################
#
# function sg_stop_srdf() is called by master control script
# during metrocluster srdf package stop.
#
#######################################################################


function sg_stop_srdf
{
    sg_log 0 "Stopping Metrocluster SRDF Package"
    # no op
    to_exit=0
}



#######################################################################
#
# function sg_validate_srdf() is called by master control script
# during metrocluster srdf configuration validation.
#
#######################################################################

function sg_validate_srdf
{
    to_exit=0
    typeset envfile;
    typeset samefile;
    typeset count;
    find_module_version "dts/mcsrdf"
    version=$?

    sg_log 0 "Validating Metrocluster Configuration"

    let version=0

    find_module_version "dts/srdf"
    version=$?

    if [ $version -gt 0 ]; then
        if [[ ! -d $SRDF_DTS_PKG_DIR ]]; then
            mkdir -p -m 755 $SRDF_DTS_PKG_DIR
            if [ $? -eq 0 ]; then
                sg_log 0 "Creation of \"$SRDF_DTS_PKG_DIR\" successful"
            else
                echo "ERROR:Failed to create \"$SRDF_DTS_PKG_DIR\" "
                to_exit=1
            fi
        fi
    fi

    if [[ ! -d $SRDF_DTS_PKG_DIR ]]; then
        echo "ERROR:  \"$SRDF_DTS_PKG_DIR\" directory does not exist"
        to_exit=1
    fi

    if [[ ! -f "$SRDF_PATH/symcli" ]]; then
        echo "ERROR: SYMCLI bin directory is not found."
        echo "INFO: Ensure that SYMCLI bin directory path is mentioned in PATH attribute."
        to_exit=1
        exit $to_exit
    fi


    if [[ $SRDF_CONSISTENCYGROUPS == '0' ]]; then
############check dg exists###################################
        output=`$SRDF_PATH/symdg show -output XML $SRDF_DEVICE_GROUP 2>&1 | grep "<mode>" | awk -F'>' '{print $2}' | awk -F'<' '{print $1}'`
        if [[ $output == 'Synchronous' || $output == 'Asynchronous'  ]]; then
            tempMode=""
            if [[ $SRDF_RDF_MODE == 'sync'  ]]; then
                tempMode="Synchronous"
            else
                tempMode="Asynchronous"
            fi
            if [[ $output != $tempMode  ]]; then
                echo "ERROR: RDF_MODE value $SRDF_RDF_MODE does not match the Device Group mode value $tempMode"
                to_exit=1
                exit $to_exit
            fi
        else
            echo "ERROR: Device Group $SRDF_DEVICE_GROUP does not exist or Device Group $SRDF_DEVICE_GROUP is set with Unsupported Replication Mode $output"
            to_exit=1
            exit $to_exit
        fi
    else
############check cg exists###################################
        output=`$SRDF_PATH/symcg show -output XML $SRDF_DEVICE_GROUP | grep "<mode>" | awk -F'>' '{print $2}' | awk -F'<' '{print $1}'`
        if [[ $output == 'Synchronous' || $output == 'Asynchronous'  ]]; then
            tempMode=""
            if [[ $SRDF_RDF_MODE == 'sync'  ]]; then
                tempMode="Synchronous"
            else
                tempMode="Asynchronous"
            fi
            if [[ $output != $tempMode  ]]; then
                echo "ERROR: RDF_MODE value $SRDF_RDF_MODE does not match the Consistency Group mode value $tempMode"
                to_exit=1
                exit $to_exit
            fi
        else
            echo "ERROR: Consistency Group $SRDF_DEVICE_GROUP does not exist or Consistency Group $SRDF_DEVICE_GROUP is set with Unsupported Replication Mode $output"
            to_exit=1
            exit $to_exit
        fi
    fi

# VMFS verification
    if (( ${#SG_VMDK_FILE_NAME[*]} > 0 )); then
        verify_vmfs_details
        if [[ $? -ne 0 ]]; then
           to_exit=1
        fi
    else
        validate_rep #validate vgs dgs 
        if [[ $? -ne 0 ]]; then
            to_exit=1
        fi
    fi

# check whether notification types are configured when the SRDF Monitor service is configured
    for cmd in "${SG_SERVICE_CMD[@]}";do
        echo $cmd | grep "$SGSBIN/DRSRDFMon $SG_PACKAGE_NAME" >> /dev/null
        if [[ $? -eq 0 ]]; then
            service_name=${SG_SERVICE_NAME[$i]}
            break
        fi
        i=$i+1
    done

    if [[ "a${service_name}" != "a" ]]; then
        if [[ "a${SRDF_DR_NOTIFICATION_CHOICES}" = "a" ]]; then
            echo "ERROR: SRDF Device Group Monitor requires the parameter"
            echo "dts/srdf/DR_NOTIFICATION_CHOICES. Please specify appropriate values for it."
            to_exit=1
        fi
    fi

# check if the notification types specified are valid
    for notif in $SRDF_DR_NOTIFICATION_CHOICES;do
        if [[ $notif != "EMAIL" && $notif != "SYSLOG" && $notif != "CONSOLE" && $notif != "OPC" ]]; then
            echo "ERROR: Invalid Notification Type $notif specified."
            to_exit=1
        fi
    done

#check if EMAIL is mentioned in the Notification Choices and email
#addresses are mentioned as part of Package configuration.

    echo "${SRDF_DR_NOTIFICATION_CHOICES}" | grep "EMAIL" >>/dev/null
    if [[ $? -eq 0 ]]; then
        echo "$SG_EMAIL_ID[0]" | grep "@" >>/dev/null
        if [[ $? -ne 0 ]]; then
            echo "ERROR: EMAIL Notification type is specified but EMAIL addresses are not configured"
            to_exit=1
        fi
    fi
    exit $to_exit
}


############################################################
# Function to verify whether service restart is needed
# or not.
# Return values:
#   0 - if restart is needed
#   1 - if restart is not needed
#   2 - if sgversion is less than 11.20.10 online reconfig
#       will bring down package
############################################################

function verify_service_restart
{
SORT=/bin/sort
    # Cluster verification and restricting online modification
    # is only supported if Serviceguard version is
    # A.11.20.20 or later
    if [[ "$sg_version" > "11.20.20" || "$sg_version" = "11.20.20" ]];  then
            cur_email_id=( $(
                for list in "${SG_EMAIL_ID[@]}";do
                    echo "$list"
                done | $SORT) )

            prev_email_id=( $(
                for list in "${PREV_SG_EMAIL_ID[@]}";do
                    echo "$list"
                done | $SORT) )
                if [[ "${SRDF_DR_NOTIFICATION_CHOICES}" != "${PREV_SRDF_DR_NOTIFICATION_CHOICES}" || "${cur_email_id[*]}" != "${prev_email_id[*]}" || "${SRDF_RDF_MODE}" != "${PREV_SRDF_RDF_MODE}" || "${SRDF_DEVICE_GROUP}" != "${PREV_SRDF_DEVICE_GROUP}" || "${SRDF_CONSISTENCYGROUPS}" != "${PREV_SRDF_CONSISTENCYGROUPS}" || "${SRDF_PATH}" != "${PREV_SRDF_PATH}" ]]; then
                    all_srv_restart=0
                else
                    all_srv_restart=1
            fi
    else
       all_srv_restart=2
    fi
return $all_srv_restart
}



#######################################################################
#
# function sg_reconfigure_start() is called by master control script
# during metrocluster srdf configuration validation.
#
#######################################################################

function sg_reconfigure_start
{
   to_exit=0
   verify_service_restart
   service_restart_temp=$?
   if [[ $service_restart_temp -eq 2 ]]; then
#exiting as online reconfiguration on SG version 11.20.10 or below will bring
#down the package if attempted
    to_exit=0
   elif [[ $service_restart_temp -eq 1 ]]; then
    sg_log 0 "Executing reconfigure start for SRDF Device Group Monitor service"
    sg_log 0 "Detected no change in parameters related to SRDF Device Group Monitor service"
    to_exit=0
   else
    sg_log 0 "Executing reconfigure start for SRDF Device Group  Monitor service"
    for cmd in "${SG_SERVICE_CMD[@]}";do
        echo $cmd | grep "$SGSBIN/DRSRDFMon $SG_PACKAGE_NAME" >> /dev/null
        if [[ $? -eq 0 ]]; then
            service_name=${SG_SERVICE_NAME[$i]}
            service_cmd=${SG_SERVICE_CMD[$i]}
            break
        fi
        i=$i+1
    done
    if [[ "a${service_name}" != "a" ]]; then
        ${SGSBIN}/cmrunserv ${service_name} "$service_cmd"
        if [[ $? -ne 0 ]]; then
            sg_log 0 "Unable to start SRDF Device Group Monitor service"
        fi
        to_exit=0
    fi
   fi
   return $to_exit
}



#######################################################################
#
# function sg_reconfigure_stop() is called by master control script
# during metrocluster SRDF configuration validation.
#
#######################################################################

function sg_reconfigure_stop
{

   to_exit=0
   verify_service_restart
   service_restart_temp=$?
   if [[ $service_restart_temp -eq 2 ]]; then
#exiting as online reconfiguration on SG version 11.20.10 or below will bring
#down the package if attempted
    to_exit=0
   elif [[ $service_restart_temp -eq 1 ]]; then
    sg_log 0 "Executing reconfigure stop for SRDF Device Group  Monitor service"
    sg_log 0 "Detected no change in parameters related to SRDF Device Group Monitor service"
    to_exit=0
   else
    sg_log 0 "Executing reconfigure stop for SRDF Device Group Monitor service"
    for cmd in "${PREV_SG_SERVICE_CMD[@]}";do
        echo $cmd | grep "$SGSBIN/DRSRDFMon $SG_PACKAGE_NAME" >> /dev/null
        if [[ $? -eq 0 ]]; then
                service_name=${PREV_SG_SERVICE_NAME[$i]}
                break
            fi
            i=$i+1
    done
    if [[ "a${service_name}" != "a" ]]; then
        ${SGSBIN}/cmhaltserv ${service_name}
        if [[ $? -ne 0 ]]; then
            sg_log 0 "Unable to stop SRDF Device Group Monitor service"
        fi
        to_exit=0
    fi
   fi
   return $to_exit

}


################
# main routine
################

#
# Module script must be specified with three required entry points:
# start, stop, and validate.
#
# The variable to_exit indicates the success or failure of the entry
# point.
#


typeset to_exit=0;



case ${1} in
            start) sg_start_srdf;;
             stop) sg_stop_srdf;;
         validate) sg_validate_srdf ;;
reconfigure_start) sg_reconfigure_start;;
 reconfigure_stop) sg_reconfigure_stop;;

esac
#
# Exit to indicate success/failure to the master_control_script.sh
#

exit $to_exit
